@@ -10,7 +10,7 @@ is.formula <- function(x){
 # output of correlations suggested by Michael Tiefelsdorf)
 #
 
-print.sarlm <- function(x, ...)
+print.Sarlm <- function(x, ...)
 {
 #FIXME
        if (x$type == "error") if (isTRUE(all.equal(x$lambda, x$interval[1])) ||
@@ -29,60 +29,20 @@ print.sarlm <- function(x, ...)
 	invisible(x)
 }
 
-summary.sarlm <- function(object, correlation = FALSE, Nagelkerke=FALSE,
+summary.Sarlm <- function(object, correlation = FALSE, Nagelkerke=FALSE,
  Hausman=FALSE, adj.se=FALSE, ...)
 {
 #FIXME
-	adj <- NULL
-	if (object$type == "error" || ((object$type == "lag" || 
-		object$type == "mixed" || object$type == "sac" || 
-                object$type == "sacmixed") && object$ase)) {
-		object$coeftitle <- "(asymptotic standard errors)"
-                SE <- object$rest.se
-                if (adj.se) {
-                    N <- length(residuals(object))
-                    adj <- N/(N-(length(object$coefficients)))
-                    SE <- sqrt((SE^2) * adj)
-                }
-		object$Coef <- cbind(object$coefficients, SE, 
-			object$coefficients/SE,
-			2*(1-pnorm(abs(object$coefficients/SE))))
-		colnames(object$Coef) <- c("Estimate", "Std. Error", 
-			ifelse(adj.se, "t value", "z value"), "Pr(>|z|)")
-	} else {
-	    # intercept-only bug fix Larry Layne 20060404
-            if (!is.null(object$rest.se)) {
-		object$coeftitle <- "(numerical Hessian approximate standard errors)"
-                SE <- object$rest.se
-                if (adj.se) {
-                    N <- length(residuals(object))
-                    adj <- N/(N-(length(object$coefficients)))
-                    SE <- sqrt((SE^2) * adj)
-                }
-		object$Coef <- cbind(object$coefficients, SE, 
-			object$coefficients/SE,
-			2*(1-pnorm(abs(object$coefficients/SE))))
-		colnames(object$Coef) <- c("Estimate", "Std. Error", 
-			ifelse(adj.se, "t value", "z value"), "Pr(>|z|)")
-	        rownames(object$Coef) <- names(object$coefficients)
-              }
-	}
-        object$adj.se <- adj
-
-        if (Nagelkerke) {
-            nk <- NK.sarlm(object)
-            if (!is.null(nk)) object$NK <- nk
-        }
         if (Hausman && object$type == "error" && !is.null(object$Hcov)) {
                 object$Haus <- Hausman.test(object)
         }
 	if (object$type == "error") {
-		object$Wald1 <- Wald1.sarlm(object)
+		object$Wald1 <- Wald1.Sarlm(object)
 		if (correlation) {
                         oresvar <- object$resvar
                         ctext <- "Correlation of coefficients"
                         if (is.null(oresvar) || is.logical(oresvar) || 
-                            class(oresvar) == "try-error") {
+                            inherits(oresvar, "try-error")) {
                             oresvar <- object$fdHess
                             ctext <- ifelse(object$insert,
                                 "Approximate correlation of coefficients",
@@ -95,12 +55,12 @@ summary.sarlm <- function(object, correlation = FALSE, Nagelkerke=FALSE,
                         object$correltext <- ctext
 		}
 	} else if (object$type != "error") {
-		object$Wald1 <- Wald1.sarlm(object)
+		object$Wald1 <- Wald1.Sarlm(object)
 		if (correlation) {
                         oresvar <- object$resvar
                         ctext <- "Correlation of coefficients"
                         if (is.null(oresvar) || is.logical(oresvar) || 
-                            class(oresvar) == "try-error") {
+                            inherits(oresvar, "try-error")) {
                             oresvar <- object$fdHess
                             ctext <- "Approximate correlation of coefficients"
                         }
@@ -111,12 +71,57 @@ summary.sarlm <- function(object, correlation = FALSE, Nagelkerke=FALSE,
                         object$correltext <- ctext
 		}
         }
-	object$LR1 <- LR1.sarlm(object)
+	object$LR1 <- LR1.Sarlm(object)
 
-	structure(object, class=c("summary.sarlm", class(object)))
+	adj <- NULL
+	if (object$type == "error" || ((object$type == "lag" || 
+		object$type == "mixed" || object$type == "sac" || 
+                object$type == "sacmixed") && object$ase)) {
+		object$coeftitle <- "(asymptotic standard errors)"
+                SE <- object$rest.se
+                if (adj.se) {
+                    N <- length(residuals(object))
+                    adj <- N/(N-(length(object$coefficients)))
+                    SE <- sqrt((SE^2) * adj)
+                }
+#                varnames <- names(object$coefficients)
+		object$Coef <- cbind(object$coefficients, SE, 
+			object$coefficients/SE,
+			2*(1-pnorm(abs(object$coefficients/SE))))
+		colnames(object$Coef) <- c("Estimate", "Std. Error", 
+			ifelse(adj.se, "t value", "z value"), "Pr(>|z|)")
+	        rownames(object$Coef) <- names(object$coefficients)
+	} else {
+	    # intercept-only bug fix Larry Layne 20060404
+            if (!is.null(object$rest.se)) {
+		object$coeftitle <- "(numerical Hessian approximate standard errors)"
+                SE <- object$rest.se
+                if (adj.se) {
+                    N <- length(residuals(object))
+                    adj <- N/(N-(length(object$coefficients)))
+                    SE <- sqrt((SE^2) * adj)
+                }
+#                varnames <- names(object$coefficients)
+		object$Coef <- cbind(object$coefficients, SE, 
+			object$coefficients/SE,
+			2*(1-pnorm(abs(object$coefficients/SE))))
+		colnames(object$Coef) <- c("Estimate", "Std. Error", 
+			ifelse(adj.se, "t value", "z value"), "Pr(>|z|)")
+	        rownames(object$Coef) <- names(object$coefficients)
+              }
+	}
+# temporary fix for broom 210312
+#        object$Coef <- object$coefficients
+        object$adj.se <- adj
+
+        if (Nagelkerke) {
+            nk <- NK.Sarlm(object)
+            if (!is.null(nk)) object$NK <- nk
+        }
+	structure(object, class=c("summary.Sarlm", class(object)))
 }
 
-print.summary.sarlm <- function(x, digits = max(5, .Options$digits - 3),
+print.summary.Sarlm <- function(x, digits = max(5, .Options$digits - 3),
 	signif.stars = FALSE, ...)
 {
 	cat("\nCall:", deparse(x$call),	sep = "", fill=TRUE)
@@ -156,7 +161,7 @@ print.summary.sarlm <- function(x, digits = max(5, .Options$digits - 3),
 	    printCoefmat(coefs, signif.stars=signif.stars, digits=digits,
 		na.print="NA")
 	}
-#	res <- LR.sarlm(x, x$lm.model)
+
 	res <- x$LR1
         pref <- ifelse(x$ase, "Asymptotic", "Approximate (numerical Hessian)")
 	if (x$type == "error") {
@@ -280,6 +285,8 @@ print.summary.sarlm <- function(x, digits = max(5, .Options$digits - 3),
         invisible(x)
 }
 
+coef.summary.Sarlm <- function(object, ...) object$Coef
+
 getVmate <- function(coefs, env, s2, trs, tol.solve=1.0e-10, optim=FALSE,
     optimM="optimHess") {
     if (optim) {
